<?php
/**
 * This file shares content automatically.
 *
 * @package    Nelio_Content
 * @subpackage Nelio_Content/includes
 * @author     David Aguilera <david.aguilera@neliosoftware.com>
 * @since      1.3.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}//end if

/**
 * This class implements all the functions used for sharing content automatically on social media.
 *
 * @package    Nelio_Content
 * @subpackage Nelio_Content/includes
 * @author     David Aguilera <david.aguilera@neliosoftware.com>
 * @since      1.3.0
 */
class Nelio_Content_Auto_Sharer {

	/**
	 * The single instance of this class.
	 *
	 * @since  1.3.0
	 * @access protected
	 * @var    int
	 */
	const MAX_SHARES_PER_DAY = 30;

	/**
	 * The single instance of this class.
	 *
	 * @since  1.3.0
	 * @access protected
	 * @var    Nelio_Content_Auto_Sharer
	 */
	protected static $_instance;

	/**
	 * Cloning instances of this class is forbidden.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	public function __clone() {

		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'nelio-content' ), '1.3.0' ); // @codingStandardsIgnoreLine

	}//end __clone()

	/**
	 * Unserializing instances of this class is forbidden.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	public function __wakeup() {

		_doing_it_wrong( __FUNCTION__, __( 'Cheatin&#8217; huh?', 'nelio-content' ), '1.3.0' ); // @codingStandardsIgnoreLine

	}//end __wakeup()

	/**
	 * Returns the single instance of this class.
	 *
	 * @return Nelio_Content_Auto_Sharer the single instance of this class.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	public static function instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}//end if

		return self::$_instance;

	}//end instance()

	/**
	 * Create or destroy WordPress cron tasks to generate social messages automatically.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	public function manage_cron_tasks() {

		$task_name = 'nc_social_automations_schedule_week';

		if ( nc_is_addon_enabled( 'social-automations' ) ) {

			$time = sprintf(
				'%02d:%02d:00',
				mt_rand( 0, 4 ),
				mt_rand( 0, 59 )
			);
			$today = date( 'Y-m-d', time() ) . 'T' . $time;
			$tomorrow = strtotime( $today . ' +1 day' );

			add_action( $task_name, array( $this, 'schedule_week' ) );
			if ( ! wp_next_scheduled( $task_name ) ) {
				wp_schedule_event( $tomorrow, 'daily', $task_name );
			}//end if

		} else {

			$timestamp = wp_next_scheduled( $task_name );
			wp_unschedule_event( $timestamp, $task_name );

		}//end if

	}//end manage_cron_tasks()


	/**
	 * This function generates social messages for the next two weeks.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	public function schedule_week() {

		// Get the day in which the current week ends.
		$now = time();
		$today = date( 'Y-m-d', $now );

		$end_of_week = get_option( 'start_of_week', 0 ) - 1;
		if ( 0 > $end_of_week ) {
			$end_of_week = 6;
		}//end if

		$days_to_end_of_week = $end_of_week - date( 'w', $now );
		if ( 0 > $days_to_end_of_week ) {
			$days_to_end_of_week = 7 + $days_to_end_of_week;
		}//end if

		$end_of_week = $today;
		if ( $days_to_end_of_week ) {
			$end_of_week = date( 'Y-m-d', strtotime( $today . ' +' . $days_to_end_of_week . ' days' ) );
		}//end if

		// Get the last day we have to schedule.
		$last_day_to_schedule = date( 'Y-m-d', strtotime( $end_of_week . ' +7 days' ) );

		// Schedule all required days.
		$last_day_scheduled = get_option( 'nc_reshare_last_day', $today );

		if ( strcmp( $last_day_scheduled, $today ) < 0 ) {
			$last_day_scheduled = $today;
		}//end if

		$guard = 0;
		while ( strcmp( $last_day_scheduled, $last_day_to_schedule ) < 0 ) {

			$posts = $this->get_posts_for_resharing( 7 );

			if ( empty( $posts ) ) {
				return;
			}//end if

			while ( count( $posts ) < 7 ) {
				shuffle( $posts );
				array_push( $posts, $posts[0] );
			}//end while

			$posts_per_day = $this->array_split( $posts, 7 );
			foreach ( $posts_per_day as $posts ) {

				$last_day_scheduled = date( 'Y-m-d', strtotime( $last_day_scheduled . ' +1 day' ) );
				$this->schedule_day( $last_day_scheduled, $posts );

				if ( $last_day_scheduled === $last_day_to_schedule ) {
					break;
				}//end if

			}//end foreach

			++$guard;
			if ( 3 < $guard ) {
				break;
			}//end if

		}//end while

		update_option( 'nc_reshare_last_day', $last_day_scheduled );

	}//end schedule_week()

	/**
	 * This function requests the cloud to generate all the messages for a given
	 * day, using the given list of posts.
	 *
	 * @param string day   day to schedule.
	 * @param array  posts list of posts used to "fill" the day.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	public function schedule_day( $day, $posts ) {

		$settings = Nelio_Content_Settings::instance();
		$data = array(
			'method'    => 'POST',
			'timeout'   => 30,
			'sslverify' => ! $settings->get( 'uses_proxy' ),
			'headers'   => array(
				'Authorization' => 'Bearer ' . nc_generate_api_auth_token(),
				'accept'        => 'application/json',
				'content-type'  => 'application/json',
			),
			'body' => wp_json_encode( array(
				'day'   => $day,
				'posts' => $posts,
			) ),
		);

		$url = sprintf(
			nc_get_api_url( '/site/%s/social/auto', 'wp' ),
			nc_get_site_id()
		);
		wp_remote_request( $url, $data );

	}//end schedule_day()

	/**
	 * This function retrieves the list of posts to share on a week.
	 *
	 * @param int $num_of_days Optional. Default: 1.
	 *
	 * @return array the list of posts to share on a week.
	 *
	 * @since  1.3.0
	 * @access private
	 */
	private function get_posts_for_resharing( $num_of_days ) {

		$result = array();
		$list_of_args = $this->get_list_of_args();

		$total_posts = $num_of_days * self::MAX_SHARES_PER_DAY;
		$posts_per_block = ceil( $total_posts / count( $list_of_args ) );

		foreach ( $list_of_args as $index => $args ) {

			$args['post__not_in'] = wp_list_pluck( $result, 'id' );

			if ( $index < count( $list_of_args ) - 1 ) {
				$args['posts_per_page'] = $posts_per_block;
			} else {
				$args['posts_per_page'] = $total_posts - count( $result );
			}//end if

			$query = new WP_Query( $args );

			while ( $query->have_posts() ) {
				$query->the_post();
				$aux = $this->post_to_json();
				unset( $aux['content'] );
				array_push( $result, $aux );
			}//end while

			wp_reset_postdata();

		}//end foreach

		shuffle( $result );
		return $result;

	}//end get_posts_for_resharing()

	/**
	 * This function prepares the list of WPDB args to select which posts will be reshared.
	 *
	 * @return array the list of WPDB args to select which posts will be reshared.
	 *
	 * @since  1.3.0
	 * @access private
	 */
	private function get_list_of_args() {

		$settings = Nelio_Content_Settings::instance();
		$enabled_post_types = $settings->get( 'calendar_post_types' );

		// 1. Get top posts by pageviews.
		$meta_name = '_nc_pageviews_total_' . $settings->get( 'google_analytics_view' );
		$meta_value = $this->get_meta_value_threshold( $meta_name );

		$pageview_args = array(
			'post_status' => 'publish',
			'post_type'   => $enabled_post_types,
			'orderby'     => 'rand(' . time() . ')',
			'meta_query'  => array(
				'relation' => 'AND',
				array(
					'key'     => $meta_name,
					'value'   => $meta_value,
					'compare' => '>=',
				),
				array(
					'key'     => '_nc_exclude_from_reshare',
					'compare' => 'NOT EXISTS',
				),
			),
		);

		// 2. Get top posts by shares.
		$meta_name = '_nc_engagement_total';
		$meta_value = $this->get_meta_value_threshold( $meta_name );

		$engagement_args = array(
			'post_status' => 'publish',
			'post_type'   => $enabled_post_types,
			'orderby'     => 'rand(' . time() . ')',
			'meta_query'  => array(
				'relation' => 'AND',
				array(
					'key'     => $meta_name,
					'value'   => $meta_value,
					'compare' => '>=',
				),
				array(
					'key'     => '_nc_exclude_from_reshare',
					'compare' => 'NOT EXISTS',
				),
			),
		);

		// 3. Get posts from last three months.
		$three_months_args = array(
			'post_status' => 'publish',
			'post_type'   => $enabled_post_types,
			'orderby'     => 'rand(' . time() . ')',
			'date_query'     => array(
				array(
					'column' => 'post_date_gmt',
					'before' => '1 month ago',
				),
				array(
					'column' => 'post_date_gmt',
					'after'  => '3 months ago',
				),
			),
			'meta_query'  => array(
				array(
					'key'     => '_nc_exclude_from_reshare',
					'compare' => 'NOT EXISTS',
				),
			),
		);

		// 4. Get posts from last year.
		$last_year_args = array(
			'post_status' => 'publish',
			'post_type'   => $enabled_post_types,
			'orderby'     => 'rand(' . time() . ')',
			'date_query'     => array(
				array(
					'column' => 'post_date_gmt',
					'before' => '3 month ago',
				),
				array(
					'column' => 'post_date_gmt',
					'after'  => '1 year ago',
				),
			),
			'meta_query'  => array(
				array(
					'key'     => '_nc_exclude_from_reshare',
					'compare' => 'NOT EXISTS',
				),
			),
		);

		// 5. Get posts from last two years.
		$two_year_args = array(
			'post_status' => 'publish',
			'post_type'   => $enabled_post_types,
			'orderby'     => 'rand(' . time() . ')',
			'date_query'     => array(
				array(
					'column' => 'post_date_gmt',
					'before' => '1 year ago',
				),
				array(
					'column' => 'post_date_gmt',
					'after'  => '2 years ago',
				),
			),
			'meta_query'  => array(
				array(
					'key'     => '_nc_exclude_from_reshare',
					'compare' => 'NOT EXISTS',
				),
			),
		);

		// 6. Get other posts.
		$default_args = array(
			'post_status' => 'publish',
			'post_type'   => $enabled_post_types,
			'orderby'     => 'rand(' . time() . ')',
			'meta_query'  => array(
				array(
					'key'     => '_nc_exclude_from_reshare',
					'compare' => 'NOT EXISTS',
				),
			),
		);

		return array(
			$pageview_args,
			$engagement_args,
			$three_months_args,
			$last_year_args,
			$two_year_args,
			$default_args,
		);

	}//end get_list_of_args()

	/**
	 * This function returns the value of a given meta.
	 *
	 * @param string $meta_name the meta whose value we want.
	 *
	 * @return int the value of the given meta.
	 *
	 * @since  1.3.0
	 * @access private
	 */
	private function get_meta_value_threshold( $meta_name ) {

		$meta_value = 1;

		// Get number of pages.
		$args = array(
			'posts_per_page' => 1,
			'meta_key'       => $meta_name,
			'orderby'        => 'meta_value_num',
			'order'          => 'desc',
		);
		$query = new WP_Query( $args );
		$num_top_posts = $query->max_num_pages;
		wp_reset_postdata();

		// Get the "threshold" post
		$last_good_post = min( $num_top_posts, 250 );
		$args['paged'] = $last_good_post;
		$query = new WP_Query( $args );
		if ( $query->have_posts() ) {
			$query->the_post();
			$meta_value = get_post_meta( get_the_ID(), $meta_name, true );
		}//end if
		wp_reset_postdata();

		if ( 0 >= $meta_value ) {
			return 1;
		} else {
			return $meta_value;
		}//end if

	}//end meta_value_threshold()

	/**
	 * This function creates an AWS-ready post object.
	 *
	 * @param integer $post_id Optional. The ID of the post we want to stringify.
	 *
	 * @return array an AWS-ready post object.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	public function post_to_json( $post_id = 0 ) {

		global $post;

		// Load some settings.
		$settings = Nelio_Content_Settings::instance();

		if ( $post_id > 0 ) {

			$my_query = new WP_Query( array(
				'p'         => $post_id,
				'post_type' => $settings->get( 'calendar_post_types' ),
			) );
			if ( ! $my_query->have_posts() ) {
				return false;
			}//end if

			$my_query->the_post();

		}//end if

		// Get the excerpt (if any).
		if ( has_excerpt() ) {
			$excerpt = get_the_excerpt();
		} else {
			$excerpt = '';
		}//end if

		// Get the appropriate permalink.
		$permalink = get_permalink();
		if ( 'publish' !== get_post_status() ) {
			$aux = clone $post;
			$aux->post_status = 'publish';
			if ( empty( $aux->post_name ) ) {
				$aux->post_name = sanitize_title( $aux->post_title, $aux->ID );
			}//end if
			$aux->post_name = wp_unique_post_slug( $aux->post_name, $aux->ID, 'publish', $aux->post_type, $aux->post_parent );
			$permalink = get_permalink( $aux );
		}//end if

		// TODO. Mirar línia 181 de wp-admin/includes/meta-boxes.php.
		$post_date = get_post_time( 'c', true, get_the_ID() );
		if ( substr( $post_date, 0, 1 ) === '-' ) {
			$post_date = 'none';
		}//end if

		$result = array(
			'id'               => get_the_ID(),
			'date'             => $post_date,
			'title'            => html_entity_decode( wp_strip_all_tags( get_the_title() ), ENT_HTML5 ),
			'excerpt'          => html_entity_decode( wp_strip_all_tags( $excerpt ), ENT_HTML5 ),
			'author'           => get_the_author_meta( 'ID' ),
			'categories'       => $this->get_categories(),
			'content'          => apply_filters( 'the_content', get_the_content() ),
			'tags'             => $this->get_tags(),
			'images'           => $this->get_images(),
			'featuredImage'    => $this->get_featured_image(),
			'permalink'        => $permalink,
			'status'           => get_post_status(),
			'timezone'         => nc_get_timezone(),
		);

		if ( $post_id > 0 ) {
			wp_reset_postdata();
		}//end if

		return $result;

	}//end post_to_json()

	/**
	 * Returns the list of categories.
	 *
	 * @return array the list of categories.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	private function get_categories() {

		if ( 'post' !== get_post_type() ) {
			return array();
		}//end if

		$helper = Nelio_Content_Post_Helper::instance();

		$aux = get_the_category();
		$processed_categories = array();
		$categories = array();

		foreach ( $aux as $cat ) {

			$root_cat = $helper->find_root_category( $cat->term_id );
			if ( $root_cat && ! in_array( $root_cat->term_id, $processed_categories, true ) ) {
				array_push( $processed_categories, $root_cat->term_id );
				array_push( $categories, $root_cat->slug );
			}//end if

		}//end foreach

		return $categories;

	}//end get_categories()

	/**
	 * Returns the list of tags.
	 *
	 * @return array the list of tags.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	private function get_tags() {

		if ( 'post' !== get_post_type() ) {
			return array();
		}//end if

		$tags = get_the_tags();
		if ( is_array( $tags ) && count( $tags ) > 0 ) {
			$tags = wp_list_pluck( get_the_tags(), 'name' );
		} else {
			$tags = array();
		}//end if

		foreach ( $tags as $i => $tag ) {
			$tags[ $i ] = preg_replace( '/ /', '', ucwords( $tag ) );
		}//end foreach

		return $tags;

	}//end get_tags()

	/**
	 * Returns the list of images.
	 *
	 * @return array the list of images.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	private function get_featured_image() {

		// Recover featured image.
		$aux = Nelio_Content_External_Featured_Image_Helper::instance();
		$featured_image = $aux->get_external_featured_image( get_the_ID() );

		if ( empty( $featured_image ) && get_post_thumbnail_id() ) {
			$featured_image = wp_get_attachment_url( get_post_thumbnail_id() );
		}//end if

		$settings = Nelio_Content_Settings::instance();
		$position = $settings->get( 'auto_feat_image' );
		if ( empty( $featured_image ) && 'disabled' !== $position ) {
			$featured_image = $aux->get_auto_featured_image( get_the_ID(), $position );
		}//end if

		if ( empty( $featured_image ) || ! $featured_image ) {
			$featured_image = 'none';
		}//end if

		return $featured_image;

	}//end get_featured_image()

	/**
	 * Returns the list of images.
	 *
	 * @return array the list of images.
	 *
	 * @since  1.3.0
	 * @access public
	 */
	private function get_images() {

		$content = get_the_content();
		preg_match_all( '/<img[^>]+>/i',$content, $matches );

		$result = array();
		foreach ( $matches[0] as $img ) {

			preg_match_all( '/src=("[^"]*"|\'[^\']*\')/i', $img, $aux );

			if ( count( $aux ) <= 1 ) {
				continue;
			}//end if

			$url = $aux[1][0];
			$url = substr( $url, 1, strlen( $url ) - 2 );
			array_push( $result, $url );

		}//end foreach

		shuffle( $result );
		return array_slice( $result, 0, 10 );

	}//end get_images()

	/**
	 * Splits the array into multiple parts.
	 *
	 * @param array $array the array to split.
	 * @param int   $parts the number of parts.
	 *
	 * @return array an array of "parts" elements, each of which will be a
	 *               subarray of the original array.
	 *
	 * @since 1.3.0
	 * @access private
	 */
	private function array_split( $array, $parts = 1 ) {

		if ( 1 >= $parts ) {
			return $array;
		}//end if

		$index = 0;
		$result = array_fill( 0, $parts, array() );
		$max = ceil( count( $array) / $parts );
		foreach ( $array as $v ) {
			if ( count( $result[ $index ] ) >= $max ) {
				++$index;
			}//end if
			array_push( $result[ $index ], $v );
		}//end foreach

		return $result;

	}//end array_split()

}//end class
